package gemini

import (
	"gophi/core"

	"github.com/grufwub/go-errors"
)

// Gemini specific base errors
var (
	invalidTLSConfigErr = errors.NewBaseError(invalidTLSConfigErrStr)
	invalidProtocolErr  = errors.NewBaseError("")
	invalidHostPortErr  = errors.NewBaseError("")
)

// generateErrorResponse takes an error code and generates an error response byte slice
func generateErrorResponse(err *errors.Error) ([]byte, bool) {
	switch {
	case err.EqualsBase(core.ConnWriteErr):
		return nil, false // no point responding if we couldn't write
	case err.EqualsBase(core.ConnReadErr):
		return buildErrorResponse("40", statusMeta40), true
	case err.EqualsBase(core.ConnCloseErr):
		return nil, false // no point responding if we couldn't close
	case err.EqualsBase(core.MutexUpgradeErr):
		return buildErrorResponse("40", statusMeta40), true
	case err.EqualsBase(core.MutexDowngradeErr):
		return buildErrorResponse("40", statusMeta40), true
	case err.EqualsBase(core.FileOpenErr):
		return buildErrorResponse("51", statusMeta51), true
	case err.EqualsBase(core.FileStatErr):
		return buildErrorResponse("51", statusMeta51), true
	case err.EqualsBase(core.FileReadErr):
		return buildErrorResponse("51", statusMeta51), true
	case err.EqualsBase(core.FileTypeErr):
		return buildErrorResponse("51", statusMeta51), true
	case err.EqualsBase(core.DirectoryReadErr):
		return buildErrorResponse("51", statusMeta51), true
	case err.EqualsBase(core.RestrictedPathErr):
		return buildErrorResponse("51", statusMeta51), true
	case err.EqualsBase(core.InvalidRequestErr):
		return buildErrorResponse("59", statusMeta59), true
	case err.EqualsBase(core.CGIStartErr):
		return buildErrorResponse("42", statusMeta42), true
	case err.EqualsBase(core.CGIExitCodeErr):
		return buildErrorResponse("42", statusMeta42), true
	default:
		return nil, false
	}
}

func buildErrorResponse(statusCode, statusMeta string) []byte {
	return append(buildResponseHeader(statusCode, statusMeta), []byte("\n")...)
}
