package gemini

import (
	"crypto/rand"
	"crypto/tls"
	"flag"
	"gophi/core"
	"io"

	"github.com/grufwub/go-errors"
	"github.com/grufwub/go-logger"
)

func init() {
	// As part of init perform initial entropy assertion
	b := make([]byte, 1)
	_, err := io.ReadFull(rand.Reader, b)
	if err != nil {
		logger.Fatal(entropyAssertFailStr)
	}
}

// Run does as says :)
func Run() {
	// Parse gemini specific flags, then all
	certFile := flag.String(certFileFlagStr, "", certFileDescStr)
	keyFile := flag.String(keyFileFlagStr, "", keyFileDescStr)
	core.ParseFlagsAndSetup(
		"gemini",
		1965,
		func() (*core.Listener, *errors.Error) {
			// Load the supplied key pair
			cert, err := tls.LoadX509KeyPair(*certFile, *keyFile)
			if err != nil {
				return nil, errors.WrapError(invalidTLSConfigErr, err)
			}

			// Create TLS config
			config := &tls.Config{
				Certificates: []tls.Certificate{cert},
			}
			config.Rand = rand.Reader

			// Create listener!
			l, err := tls.Listen("tcp", core.Bind+":"+core.Port, config)
			if err != nil {
				return nil, errors.WrapError(core.ListenerBeginErr, err)
			}

			// Return wrapper listener
			return core.NewListener(l), nil
		},
		newFileContent,
		handleDirectory,
		handleLargeFile,
	)

	// Start!
	core.Start(serve)
}
