package gopher

import (
	"flag"
	"gophi/core"
	"net"

	"github.com/grufwub/go-errors"
)

// Run does as says :)
func Run() {
	// Parse gopher specific flags, then all
	pWidth := flag.Uint(pageWidthFlagStr, 80, pageWidthDescStr)
	footerText := flag.String(footerTextFlagStr, "Gophi, a Gopher server in Go!", footerTextDescStr)
	subgopherSizeMax := flag.Float64(subgopherSizeMaxFlagStr, 1.0, subgopherSizeMaxDescStr)
	admin := flag.String(adminFlagStr, "", adminDescStr)
	desc := flag.String(descFlagStr, "", descDescStr)
	geo := flag.String(geoFlagStr, "", geoDescStr)
	core.ParseFlagsAndSetup(
		"gopher",
		70,
		func() (*core.Listener, *errors.Error) {
			l, err := net.Listen("tcp", core.Bind+":"+core.Port)
			if err != nil {
				return nil, errors.WrapError(core.ListenerBeginErr, err)
			}
			return core.NewListener(l), nil
		},
		newFileContent,
		handleDirectory,
		handleLargeFile,
	)

	// Setup gopher specific global variables
	subgophermapSizeMax = int64(1048576.0 * *subgopherSizeMax) // convert float to megabytes
	pageWidth = int(*pWidth)
	footer = buildFooter(*footerText)

	// Add generated policy file to cache
	p := core.NewPath(core.Root, "caps.txt")
	core.SystemLog.Infof("Generating policy file %s...", p.Absolute())
	core.AddGeneratedFile(p, generateCapsTxt(*desc, *admin, *geo))

	// Start!
	core.Start(serve)
}
